class GitXplorer::GitObject
    attr_reader :name
    attr_reader :parent

    def absolute_path
        return "" if (@parent.nil?)
        return [@parent.absolute_path, @name].join
    end

    def children
        @kids ||= Array.new
        return @kids
    end

    def desc
        return ""
    end

    def exist?(path)
        return !get(path).nil?
    end

    def get(path)
        # Return self if path is nil or empty
        return self if (path.nil? || path.empty?)

        # Split path on /
        child, recurse, grandchild = path.partition("/")
        child, _, grandchild = path.partition(":") if (recurse.empty?)

        case child
        when "."
            # Same directory
            return self.get(grandchild)
        when ".."
            # Check parent if it exists
            return @parent.get(grandchild) if (@parent)

            # Otherwise we reached top-level so just keep going
            return self.get(grandchild)
        end

        # Check child if it exists
        if (has_child?(child))
            kids = children.select do |kid|
                kid.name == child
            end
            return kids[0].get(grandchild) if (!kids.empty?)
        end

        # Not found
        return nil
    end

    def get_completions(path)
        # Return all children if path is nil or empty
        return children if (path.nil? || path.empty?)

        # Split path on / or :
        child, recurse, grandchild = path.partition("/")
        if (recurse.empty?)
            child, recurse, grandchild = path.partition(":")
        end

        case child
        when "."
            # Same directory
            return get_completions(grandchild)
        when ".."
            # Check parent if it exists
            return @parent.get_completions(grandchild) if (@parent)

            # Otherwise we reached top-level so just keep going
            return get_completions(grandchild)
        end

        # Check child if it exists
        if (!recurse.empty? && has_child?(child))
            return get(child).get_completions(grandchild)
        end

        # Return any children that may match
        return children.select do |kid|
            kid.name.downcase.start_with?(child.downcase)
        end
    end

    def has_child?(name)
        return children.any? do |child|
            child.name == name
        end
    end

    def initialize(name, parent)
        @kids = nil
        @name = name
        @parent = parent
    end

    def tab_complete(color = false)
        color ||= false
        return {@name => desc}
    end

    def to_s
        return @name
    end
end

require "git_xplorer/git_object/directory"
require "git_xplorer/git_object/file"
require "git_xplorer/git_object/revision"
