class GitXplorer::GitObject::Directory < GitXplorer::GitObject
    def absolute_path
        return "" if (@parent.nil?)
        return "#{super}/"
    end

    def add(path)
        dir, _, file = path.partition("/")
        if (file.nil? || file.empty?)
            # Add file
            file = GitXplorer::GitObject::File.new(dir, self)
            @files[file.name] = file
        else
            # Add subdirectory
            if (@directories[dir].nil?)
                directory = GitXplorer::GitObject::Directory.new(
                    dir,
                    self
                )
                @directories[dir] = directory
            end
            @directories[dir].add(file)
        end
    end

    def children
        if (@kids.nil?)
            # Initialize
            @kids = Array.new

            # Get child directories first
            @kids.concat(
                @directories.values.sort do |a, b|
                    a.name.downcase <=> b.name.downcase
                end
            )

            # Get child files
            @kids.concat(
                @files.values.sort do |a, b|
                    a.name.downcase <=> b.name.downcase
                end
            )
        end

        return @kids
    end

    def desc
        return "Directory"
    end

    def initialize(name, parent)
        super(name.gsub(%r{/+$}, ""), parent)
        @directories = Hash.new
        @files = Hash.new
    end

    def tab_complete(color = false)
        color ||= false
        return {"#{@name}/" => desc}
    end

    def to_s
        return "#{@name}/"
    end
end
