require "djinni"
require "hilighter"

class ChangeDirectoryWish < Djinni::Wish
    def aliases
        return ["cd"]
    end

    def description
        return "Change to new directory"
    end

    def execute(args, djinni_env = Hash.new)
        gitx = djinni_env["gitXplorer"]
        begin
            gitx.cd(args)
            prompt = "#{gitx.pwd_short}$ ".light_white
            djinni_env["djinni_prompt"] = prompt
        rescue GitXplorer::Error => e
            puts e.message
        end
    end

    def tab_complete(input, djinni_env = Hash.new)
        gitx = djinni_env["gitXplorer"]

        _, _, partial = input.rpartition("/")
        partial = "" if (input.end_with?("/"))

        completions = Hash.new
        gitx.get_completions(input).select do |child|
            child.is_a?(GitXplorer::GitObject::Directory)
        end.each do |child|
            completions.merge!(child.tab_complete)
        end

        return [completions, partial, ""]
    end

    def usage
        puts "#{aliases.join(", ")} [directory]"
        puts "    #{description}."
    end
end
