require "djinni"

class GrepWish < Djinni::Wish
    def aliases
        return ["grep"]
    end

    def description
        return "Similar to \"git grep -HIinP PATTERN\""
    end

    def execute(args, djinni_env = Hash.new)
        gitx = djinni_env["gitXplorer"]
        args.gsub!(/^(["'])(.*)\1$/, "\\2")
        dir = gitx.pwd.gsub(%r{^[^/]+(/|$)}, "")

        if (args.empty?)
            usage
            return
        end

        begin
            gitx.search(dir, args).each do |line|
                m = line.match(/^([^:]+):([^:]+):(\d+):(.*)$/)
                if (m && (m.length == 5))
                    puts [
                        m[1].green,
                        m[2].cyan,
                        m[3].white,
                        m[4].gsub(/(#{args})/i, "\\1".black.on_white)
                    ].join(":")
                else
                    puts line
                end
            end
        rescue GitXplorer::Error => e
            puts e.message
        end
    end

    def usage
        puts "#{aliases.join(", ")} <PATTERN>"
        puts "    #{description}."
    end
end
